// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QQMLREGISTRATION_H
#define QQMLREGISTRATION_H

#include <QtCore/qglobal.h>
#include <QtQmlIntegration/qqmlintegration.h>

// satisfy configure, which warns about public headers not using those
QT_BEGIN_NAMESPACE

QT_END_NAMESPACE

#endif // QQMLREGISTRATION_H
