/* This file was generated by syncqt. */
#ifndef QT_QTQMLFORMAT_VERSION_H
#define QT_QTQMLFORMAT_VERSION_H

#define QTQMLFORMAT_VERSION_STR "6.10.2"

#define QTQMLFORMAT_VERSION 0x0601002

#endif // QT_QTQMLFORMAT_VERSION_H
