#ifndef QT_FEATURES_QuickControls2_src_quickcontrols_qtquickcontrols2_config_p_h_H
#define QT_FEATURES_QuickControls2_src_quickcontrols_qtquickcontrols2_config_p_h_H

#define QT_FEATURE_quickcontrols2_basic 1

#define QT_FEATURE_quickcontrols2_fusion 1

#define QT_FEATURE_quickcontrols2_imagine 1

#define QT_FEATURE_quickcontrols2_material 1

#define QT_FEATURE_quickcontrols2_universal 1

#define QT_FEATURE_quickcontrols2_fluentwinui3 1

#define QT_FEATURE_quickcontrols2_macos -1

#define QT_FEATURE_quickcontrols2_ios -1

#define QT_FEATURE_quickcontrols2_windows -1


#endif // QT_FEATURES_QuickControls2_src_quickcontrols_qtquickcontrols2_config_p_h_H
