/* This file was generated by syncqt. */
#ifndef QT_QTQUICKLAYOUTS_VERSION_H
#define QT_QTQUICKLAYOUTS_VERSION_H

#define QTQUICKLAYOUTS_VERSION_STR "6.10.2"

#define QTQUICKLAYOUTS_VERSION 0x0601002

#endif // QT_QTQUICKLAYOUTS_VERSION_H
